# AFLFast

## What is AFLFast

https://github.com/mboehme/aflfast

AFLFast[^ccs16] is an [AFL](https://github.com/google/AFL) based fuzzer developed by Marcel Böhme and the team. They found that most of the input generated by AFL from a given seed executes the same path, whereas other paths with interesting behaviors are less executed. By improving these behaviors, AFLFast finds the inputs that trigger vulnerability-related bugs about seven times faster than AFL. Since it is widely used as a baseline for AFL-based research, we implemented AFLFast in fuzzuf.

## CLI Usage

fuzzuf AFLFast provides the same CLI interface with fuzzuf AFL:

```bash
fuzzuf aflfast --in_dir=path/to/initial/seeds/ -- path/to/PUT @@
```

The AFLFast CLI has the same global options as the fuzzuf AFL CLI. Refer to the [AFL/algorithm_en.md](/docs/algorithms/afl/algorithm_en.md) for the available options. The AFLFast CLI does not have specific local options.
Note that the current CLI implementation does not provide the option that selects schedules described below. _FAST_ is used by default.

## Algorithm Overview

The original AFL algorithm had the problem that makes its input generation inefficient. For the seed executing a given path ![i](https://render.githubusercontent.com/render/math?math=i) to discover an input that executes the path ![j](https://render.githubusercontent.com/render/math?math=j), the amount of inputs generated from the seed (called _energy_) is an important factor. If the energy is too large, it is inefficient even the most effective method [^tse15], and if the energy is too small, it cannot find the input that executes path ![j](https://render.githubusercontent.com/render/math?math=j). AFL assigns constant energy from the seed, resulting in over/under energy assignments.

AFLFast implements five types of power schedules. The FAST power schedule assigns the energy exponentially. It assigns very low energy to the seed when the fuzzer selects the seed for the first time, and each time the seed is selected, more energy is assigned, and the number of inputs generated increases. This algorithm allows us to assign the energy needed to discover new paths efficiently.

## Power Schedules

TODO: Add power schedule description.

## References

[^ccs16]: Marcel Böhme, Van-Thuan Pham, and Abhik Roychoudhury. 2016. Coverage-based Greybox Fuzzing as Markov Chain. In Proceedings of the 23rd ACM Conference on Computer and Communications Security (CCS’16).
[^tse15]: Marcel Bohme and Soumya Paul. 2015. A Probabilistic Analysis of the Efficiency of Automated Software Testing. In IEEE Transactions on Software Engineering (TSE'15).

